// Persistence of Vision Ray Tracer Include File 
// Part of the IRTC GIP object archives 
// Version: 1.0

// Macro:  IR_ParkFence(length, closed)       
// Object: IR_ParkGate  

// Object Description: Iron fence for the perimeter of the park 
//                     and matching entrance gate.   
// Author: Ib Rasmussen 
// Email: ir@ibras.dk
// Web page: http://www.ibras.dk/
// Date (yyyy.mm.dd): 2000.01.25. 

// Origin: Fence: Left end, center of first fence post. Aligned along x-axis.
//         Gate:  Center of gate. Aligned along x-axis.
// Orientation: Y = up. 
// Dimensions: 1 unit = 1 cm 

// Includes required: colors.inc, stones1.inc
// Example: object { IR_ParkFence(3000, 1) }
//          object { IR_ParkGate } 

// Variables: 

// Comments: The fence and gate are designed to be combined into
//           a rectangular enclosure with cut-off corners, where
//           the entrances are places at a 45 deg angle to the sides.
//           The corners shall be cut 5 meters from the apex. The
//           center of the gate should be placed 2.5 meters from the
//           apex on each axis.
//           The length of the sides are rounded up to the nearest 2 m.
//           The 'closed' parameter controls whether the length of
//           fence should end with a fence post (closed = 1) or not
//           (closed = 0). The gate segments have open ends.

// Scaling ///////////////////////////////////////////////////// 
// 1 cm = 1 baseUnit 
#ifndef(baseUnit) #declare baseUnit = 1; #end 
/////// DO NOT CHANGE ANYTHING BELOW ///////////////////////////// 
// Metric 
#ifndef (m) /***meter****/ #local m  = 100 *baseUnit; #end 
#ifndef(cm) /*centimeter*/ #local cm = 1.0 *baseUnit; #end 
#ifndef(mm) /*millimeter*/ #local mm = 0.1 *baseUnit; #end 
// English 
#ifndef(in) /*inch*/ #local in = 2.54 *baseUnit; #end 
#ifndef(ft) /*foot*/ #local ft = 12*in*baseUnit; #end 
#ifndef(yd) /*yard*/ #local yd = 3*ft *baseUnit; #end 
////////////////////////////////////////////////////////////////// 

global_settings { assumed_gamma 1.0 } 

/*********************** Control Variables ***************************/

#ifndef (ShowDetail)
   #local ShowDetail=1;
#end   

// Local

   
/************************* Declarations ******************************/
 
/*-------------------------- Textures -------------------------------*/

#local VeryDarkGreen = color red 0.02 green 0.03 blue 0.02;

#local T_PFJern=texture{
   pigment { VeryDarkGreen }
#if(ShowDetail=1)
   finish {phong 0.05  phong_size 10}
   normal { bumps 0.1 scale <0.5, 0.5, 1> }
#end
}

#local T_PFBase=texture{
#if(ShowDetail=1)
   T_Stone20
#else
   pigment { Gray30 }
#end   
}
   
/*-------------------------- Objects --------------------------------*/

#declare PFPost=union {                           
   cone { <0, 0, 0>, 6, <0, 9, 0>, 2 }
   cylinder { <-1, 0, -1>, <-1, 135, -1>, 2 }
   cylinder { < 1, 0, -1>, < 1, 135, -1>, 2 }
   cylinder { <-1, 0,  1>, <-1, 135,  1>, 2 }
   cylinder { < 1, 0,  1>, < 1, 135,  1>, 2 }
   difference { 
      intersection {
         sphere { <0, -1.25, 0>, 13.25 }
         cylinder { <0, 0, 0>, <0, 12, 0>, 7 }
      }   
      torus { 7, 4 scale <1, 8/4, 1> }
      translate <0, 130, 0>            
   }                                     
   torus { 2.8, 1 translate <0, 6.5, 0> }
   torus { 2.8, 1 translate <0, 130, 0> }
   texture { T_PFJern }
}

#local PFPin=box { <-1, 5, -1>, <1, 130, 1> texture { T_PFJern } }

#local PFBar=box { <0, -1, -1.2>, <200, 1, 1.2> texture { T_PFJern } } 

#declare PFSegment=union {
   object { PFPost translate <0, 10, 0> }
   object { PFBar translate <0,  30, 0> }
   object { PFBar translate <0, 120, 0> }
#if(ShowDetail=1)
   #local xpos = 20;
   #while (xpos < 200)
      object { PFPin translate <xpos, 10, 0> }
      #local xpos = xpos + 20;
   #end          
#end    

   box { <-6.2132, -250, -15>, <206.2132, 10, 15> texture { T_PFBase } }   
}

/*------ Gate -----------*/

#local PFGateDecoPart=difference {
   cylinder { <0, 0, -0.5>, <0, 0, 0.5>, 3 }
   cylinder { <0, 0, -1.5>, <0, 0, 1.5>, 1.5 }
   cylinder { <0, -3.5, -1.5>, <0, -3.5, 1.5>, 2.5 }
}
                   
#local PFGateDeco=union {
   object { PFGateDecoPart translate <0, 3.5, 0> }
   object { PFGateDecoPart translate <0, 3.5, 0> rotate <0, 0,  90> }
   object { PFGateDecoPart translate <0, 3.5, 0> rotate <0, 0, 180> }
   object { PFGateDecoPart translate <0, 3.5, 0> rotate <0, 0, 270> }
}   
   
#local PFGate=union{
   cylinder { <-92, 0, 0>, <-92, 2, 0>, 5 }
   sphere { <-92, 4, 0>, 2.5 }
   cylinder { <-92, 4, 0>, <-92, 150, 0>, 2.5 }
   difference { 
      intersection {
         sphere { <0, -1.25, 0>, 13.25 }
         cylinder { <0, 0, 0>, <0, 12, 0>, 6 }
      }                         
      torus { 6, 3.5 scale <1, 8/3.5, 1> }
      translate <-92, 150, 0>            
   }                                     
   torus { 2.3, 0.7 translate <-92, 150, 0> }
   difference {
      cylinder { <0, -3.0192, -1.5>, <0, -3.0192, 1.5>, 159.0192 }
      cylinder { <0, -3.0192, -2.5>, <0, -3.0192, 2.5>, 156.0192 }
      plane {-x, 0 translate <-0.5, 0, 0> }
      plane { x, 0 rotate <0, 0, 16.6928> translate <0, -3.0192, 0> }
   }
   difference {
      cylinder { <-89.9647, 298.9006, -1.5>, <-89.9647, 298.9006, 1.5>, 159.0192 }
      cylinder { <-89.9647, 298.9006, -2.5>, <-89.9647, 298.9006, 2.5>, 156.0192 }
      plane { x, 0 translate <-92, 0, 0> }
      plane {-x, 0 rotate <0, 0, 16.6928> translate <0, -3.0192, 0> }    
   }                             
   box { <-3.25, 5, -1.5>, <-0.25, 155, 1.5> }
   box { <-92,   5, -1.5>, <-0.25,   8, 1.5> }  

   box { <-92,  20, -1.5>, <-0.25,  22, 1.5> }  
   box { <-92,  35, -1.5>, <-0.25,  37, 1.5> }  
   box { <-92, 113, -1.5>, <-0.25, 115, 1.5> }  
   box { <-92, 128, -1.5>, <-0.25, 130, 1.5> }  
   
   box { <-18.00, 6, -1>, <-16.00, 154, 1> }   
   box { <-32.75, 6, -1>, <-30.75, 151, 1> }  
   box { <-47.50, 6, -1>, <-45.50, 147, 1> }  
   box { <-62.25, 6, -1>, <-60.25, 144, 1> }  
   box { <-77.00, 6, -1>, <-75.00, 142, 1> }  

   object { PFGateDeco translate < -9.625, 121.5, 0> }
   object { PFGateDeco translate <-24.375, 121.5, 0> }
   object { PFGateDeco translate <-39.125, 121.5, 0> }
   object { PFGateDeco translate <-53.875, 121.5, 0> }
   object { PFGateDeco translate <-68.625, 121.5, 0> }
   object { PFGateDeco translate <-83.375, 121.5, 0> }
   
   cylinder { <-92, 132, 0>, <-92, 137, 0>, 3.5 }
   box { <-110, 132, -1>, <-92, 137, 1> }
   
   texture { T_PFJern }
}

#local PFGColumnCutout=cylinder { <0, 13, 0>, <0, 157, 0>, 3.1  scale <1, 1, 0.6> }

#local PFGColumn=union{ 
   box { <-21, 0, -21>, <21, 10, 21> }
   difference {
      box { <-20, 0, -20>, <20, 170, 20> }   
      object { PFGColumnCutout translate <-15.4, 0, -20> }
      object { PFGColumnCutout translate < -7.7, 0, -20> }
      object { PFGColumnCutout translate <  0,   0, -20> }
      object { PFGColumnCutout translate <  7.7, 0, -20> }
      object { PFGColumnCutout translate < 15.4, 0, -20> }
      object { PFGColumnCutout translate <-15.4, 0,  20> }
      object { PFGColumnCutout translate < -7.7, 0,  20> }
      object { PFGColumnCutout translate <  0,   0,  20> }
      object { PFGColumnCutout translate <  7.7, 0,  20> }
      object { PFGColumnCutout translate < 15.4, 0,  20> }
   }   
   box { <-21, 160, -21>, <21, 165, 21> }
   difference {
      box { <-25, 164, -25>, <25, 170, 25> }
      cylinder { <-30, 164, -26>, <30, 164, -26>, 4 }
      cylinder { <-30, 164,  26>, <30, 164,  26>, 4 }
      cylinder { <-26, 164, -30>, <-26, 164, 30>, 4 }
      cylinder { < 26, 164, -30>, < 26, 164, 30>, 4 }
   }   
   difference {
      box { <-27, 170, -27>, <27, 200, 27> }
      plane { -y, 0 rotate <0, 0, 10> translate <-27, 176, 0> }
      plane { -y, 0 rotate <0, 0, -10> translate <27, 176, 0> }
   }
   box { <-30, -250,  -15>, <-20,  10,  15> texture { T_PFBase } }   
   box { <-30,   29, -1.2>, <-20,  31, 1.2> texture { T_PFJern } }
   box { <-30,  119, -1.2>, <-20, 121, 1.2> texture { T_PFJern } }
   texture { T_Stone20 scale 5 }
}
      
/*------------------ Assembled Objects -------------------------------*/

#macro IR_ParkFence(length, closed)
   union {
      #local xpos = 0;
      #while (xpos < length)      
         object { PFSegment translate <xpos, 0, 0> }
         #local xpos = xpos + 200;
      #end                        
      #if(closed = 1)
         object { PFPost translate <xpos, 10, 0> }
      #end
   }
   #declare IR_ParkFence_x = length + 12.4624;
   #declare IR_ParkFence_y = 402;
   #declare IR_ParkFence_z = 30;
#end

#declare IR_ParkGate=union{       
   object { PFGate }
   object { PFGate scale <-1, 1, 1> }
   object { PFGColumn translate <-122, 0, 0> }
   object { PFGColumn scale <-1, 1, 1> translate < 122, 0, 0> }
   object { PFSegment scale <-1, 1, 1> translate <-153.5534, 0, 0> }
   object { PFSegment translate <153.5534, 0, 0> }
}
#declare IR_ParkGate_x = 719.5332;
#declare IR_ParkGate_y = 181;
#declare IR_ParkGate_z = 42;
